@@ -61,6 +61,16 @@ App::App(int &argc, char ** argv)
 	if (!qgetenv("NVIM_QT_LOG").isEmpty()) {
 		qInstallMessageHandler(logger);
 	}
+
+	QByteArray stylesheet_path = qgetenv("NVIM_QT_STYLESHEET");
+	if (!stylesheet_path.isEmpty()) {
+		QFile qssfile(stylesheet_path);
+		if (qssfile.open(QIODevice::ReadOnly)) {
+			setStyleSheet(qssfile.readAll());
+		} else {
+			qWarning("Unable to open stylesheet from $NVIM_QT_STYLESHEET");
+		}
+	}
 }
 
 bool App::event(QEvent *event)
@@ -76,6 +86,11 @@ bool App::event(QEvent *event)
 
 void App::showUi(NeovimConnector *c, const QCommandLineParser& parser)
 {
+	auto opts = ShellOptions();
+	if (parser.isSet("no-ext-tabline")) {
+		opts.enable_ext_tabline = false;
+	}
+
 #ifdef NEOVIMQT_GUI_WIDGET
 	NeovimQt::Shell *win = new NeovimQt::Shell(c);
 	win->show();
@@ -87,7 +102,7 @@ void App::showUi(NeovimConnector *c, const QCommandLineParser& parser)
 		win->show();
 	}
 #else
-	NeovimQt::MainWindow *win = new NeovimQt::MainWindow(c);
+	NeovimQt::MainWindow *win = new NeovimQt::MainWindow(c, opts);
 
 	QObject::connect(instance(), SIGNAL(openFilesTriggered(const QList<QUrl>)),
 		win->shell(), SLOT(openFiles(const QList<QUrl>)));
@@ -120,8 +135,13 @@ void App::processCliOptions(QCommandLineParser &parser, const QStringList& argum
 	parser.addOption(QCommandLineOption("geometry",
 				QCoreApplication::translate("main", "Initial window geometry"),
 				QCoreApplication::translate("main", "geometry")));
+	parser.addOption(QCommandLineOption("stylesheet",
+				QCoreApplication::translate("main", "Apply qss stylesheet from file"),
+				QCoreApplication::translate("main", "stylesheet")));
 	parser.addOption(QCommandLineOption("maximized",
 				QCoreApplication::translate("main", "Maximize the window on startup")));
+	parser.addOption(QCommandLineOption("no-ext-tabline",
+				QCoreApplication::translate("main", "Disable the external GUI tabline")));
 	parser.addOption(QCommandLineOption("fullscreen",
 				QCoreApplication::translate("main", "Open the window in fullscreen on startup")));
 	parser.addOption(QCommandLineOption("embed",
@@ -140,7 +160,7 @@ void App::processCliOptions(QCommandLineParser &parser, const QStringList& argum
 
 	parser.addPositionalArgument("file",
 			QCoreApplication::translate("main", "Edit specified file(s)"), "[file...]");
-	parser.addPositionalArgument("...", "Additional arguments are fowarded to Neovim", "[-- ...]");
+	parser.addPositionalArgument("...", "Additional arguments are forwarded to Neovim", "[-- ...]");
 
 	parser.process(arguments);
 
