@@ -3,6 +3,7 @@
 
 #include <QMainWindow>
 #include <QStackedWidget>
+#include <QTabBar>
 #include "neovimconnector.h"
 #include "errorwidget.h"
 #include "shell.h"
@@ -20,7 +21,7 @@ public:
 		FullScreen,
 	};
 
-	MainWindow(NeovimConnector *, QWidget *parent=0);
+	MainWindow(NeovimConnector *, ShellOptions opts, QWidget *parent=0);
 	bool neovimAttached() const;
 	Shell* shell();
 public slots:
@@ -41,6 +42,9 @@ private slots:
 	void reconnectNeovim();
 	void showIfDelayed();
 	void neovimAttachmentChanged(bool);
+	void neovimIsUnsupported();
+	void neovimTablineUpdate(int64_t curtab, QList<Tab> tabs);
+	void changeTab(int index);
 private:
 	void init(NeovimConnector *);
         NeovimConnector *m_nvim;
@@ -48,6 +52,9 @@ private:
 	Shell *m_shell;
 	DelayedShow m_delayedShow;
 	QStackedWidget m_stack;
+	QTabBar *m_tabline;
+	QToolBar *m_tabline_bar;
+	ShellOptions m_shell_options;
 };
 
 } // Namespace
