@@ -48,7 +48,6 @@ MsgpackIODevice::MsgpackIODevice(QIODevice *dev, QObject *parent)
 :QObject(parent), m_reqid(0), m_dev(dev), m_encoding(0), m_reqHandler(0), m_error(NoError)
 {
 	qRegisterMetaType<MsgpackError>("MsgpackError");
-	qRegisterMetaType<Function::FunctionId>("Function::FunctionId");
 	msgpack_unpacker_init(&m_uk, MSGPACK_UNPACKER_INIT_BUFFER_SIZE);
 
 	if (m_dev) {
@@ -269,7 +268,7 @@ void MsgpackIODevice::dispatch(msgpack_object& req)
 		}
 		if (req.via.array.ptr[3].type != MSGPACK_OBJECT_ARRAY) {
 			qDebug() << "Invalid request: arguments MUST be an array";
-			sendError(req, tr("Paremeters must be an array"));
+			sendError(req, tr("Parameters must be an array"));
 			return;
 		}
 		dispatchRequest(req);
